﻿using System;
using System.Collections.Generic;
using System.Linq;
using JsonRpcLite.Rpc;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Interface.DBInterface;

namespace WingUserModule.Processor
{
    public static class LoginProcessorCreator
    {
        private static Dictionary<LoginProcessorTypeEnum, ILoginProcessor> _processorDic = new();

        /// <summary>
        /// 设置进程内Rpc客户端
        /// </summary>
        /// <param name="client"></param>
        public static void InitProcessorCreator(IUserDBService userServiceProxy, IAuthenticationService authenticationServiceProxy,Action<string> checkedLoginLockAction)
        {
            _processorDic.Add(LoginProcessorTypeEnum.Email, new EmailLoginProcessor(userServiceProxy, authenticationServiceProxy,checkedLoginLockAction));
            _processorDic.Add(LoginProcessorTypeEnum.Official, new OfficialLoginProcessor(userServiceProxy, authenticationServiceProxy,checkedLoginLockAction));
            _processorDic.Add(LoginProcessorTypeEnum.Phone, new PhoneLoginProcessor(userServiceProxy, authenticationServiceProxy,checkedLoginLockAction));
        }

        /// <summary>
        /// 获取登录方式
        /// </summary>
        /// <param name="processorType">登录类型</param>
        /// <returns>登录过程调用对象</returns>
        public static ILoginProcessor GetProcessor(LoginProcessorTypeEnum processorType)
        {
            _processorDic.TryGetValue(processorType, out ILoginProcessor processor);
            return processor;
        }
    }
}
