﻿using System.Threading.Tasks;
using WingUserModule.Entity;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request.DBRequest;
using System;
using WingServerCommon.Config;

namespace WingUserModule.Processor
{
    [LoginProcessorFlag(LoginProcessorTypeEnum.Official)]
    public class OfficialLoginProcessor : ILoginProcessor
    {
        private IUserDBService _userServiceProxy;
        private IAuthenticationService _authenticationServiceProxy;
        private Action<string> _checkedLoginLockAction;

        /// <summary>
        /// Init processor
        /// </summary>
        /// <param name="baseRpcService"></param>
        public OfficialLoginProcessor(IUserDBService userServiceProxy, IAuthenticationService authenticationServiceProxy,Action<string> checkedLoginLockAction)
        {
            _userServiceProxy = userServiceProxy;
            _authenticationServiceProxy = authenticationServiceProxy;
            _checkedLoginLockAction=checkedLoginLockAction;
        }

        /// <summary>
        /// 登录
        /// </summary>
        /// <param name="userLoginInfo">用户登录参数</param>
        /// <returns>Token信息</returns>
        public async Task<string> DoLoginAsync(UserLoginInfo userLoginInfo)
        {
            var officialLgoin = userLoginInfo;
            if (string.IsNullOrWhiteSpace(officialLgoin.Password))
            {
                throw new RpcException((int)CustomerRpcCode.PasswordEmpty, "Password can't be null", "Password can't be null");
            }
            UserDTO userDTO = null;
            if (!string.IsNullOrWhiteSpace(officialLgoin.UserName))
            {
                userDTO = await _userServiceProxy.FindUserByNameAsync(officialLgoin.UserName);
            }
            if (userDTO == null && !string.IsNullOrWhiteSpace(officialLgoin.Phone))
            {
                userDTO = await _userServiceProxy.FindUserByPhoneAsync(officialLgoin.Phone);
            }
            if (userDTO == null && !string.IsNullOrWhiteSpace(officialLgoin.Email))
            {
                userDTO = await _userServiceProxy.FindUserByEmailAsync(officialLgoin.Email);
            }

            if (userDTO != null)
            {
                userLoginInfo.UserCode=userDTO.UserCode;
                if (!await _userServiceProxy.VerificationPasswordAsync(userDTO.UserCode, officialLgoin.Password))
                {
                    throw new RpcException((int)CustomerRpcCode.PasswordError, "Password is wrong!", "Password is wrong!");
                }
                _checkedLoginLockAction?.Invoke(userLoginInfo.UserCode);
                await UpgradeLogintimes(userDTO.MappingTo<UserPasswordDTO>());
                var plateform = userLoginInfo.Platform;
                var loginSource = userLoginInfo.LoginSource;
                var userCode = userDTO.UserCode;
                var loginServer = ConfigurationManager.Host;//should be from userLoginInfo
                var ipValue = 0;//should be from userLoginInfo
                var token = await _authenticationServiceProxy.ApplyTokenAsync(new ApplyTokenRequest(AccountType.User,userDTO.UserName, plateform, loginSource, userCode, loginServer, ipValue));
                return token.Code;
            }
            throw new RpcException((int)CustomerRpcCode.IncorrectAccount, "Incorrect account", "Incorrect account");
        }

        private async Task<bool> UpgradeLogintimes(UserPasswordDTO userPasswordDTO)
        {
            userPasswordDTO.Logintimes += 1;
            if (userPasswordDTO.Logintimes == 1)
            {
                userPasswordDTO.UserState = UserInfoStateEnum.Activated;
            }
            return await _userServiceProxy.UpdateUserLoginInfoAsync(
                new UpdateUserLoginInfoDBRequest()
                {
                    Code=userPasswordDTO.UserCode,
                    LoginTimes=userPasswordDTO.Logintimes,
                    UserState= userPasswordDTO.UserState,
                    LoginLockInfo=new LoginLockInfoDTO()
            });
        }
    }
}
