﻿using System.Threading.Tasks;
using WingUserModule.Entity;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Request.Authentication;
using WingInterfaceLibrary.DTO.User;
using WingInterfaceLibrary.Interface.DBInterface;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.Request.DBRequest;
using System;
using WingServerCommon.Config;

namespace WingUserModule.Processor
{
    [LoginProcessorFlag(LoginProcessorTypeEnum.Phone)]
    public class PhoneLoginProcessor : ILoginProcessor
    {
        private Action<string> _checkedLoginLockAction;
        private IUserDBService _userServiceProxy;
        private IAuthenticationService _authenticationServiceProxy;

        public PhoneLoginProcessor(IUserDBService userServiceProxy, IAuthenticationService authenticationServiceProxy, Action<string> checkedLoginLockAction)
        {
            _userServiceProxy = userServiceProxy;
            _authenticationServiceProxy = authenticationServiceProxy;
            _checkedLoginLockAction = checkedLoginLockAction;
        }

        /// <summary>
        /// 登录
        /// </summary>
        /// <param name="userLoginInfo">用户登录参数</param>
        /// <returns>Token信息</returns>
        public async Task<string> DoLoginAsync(UserLoginInfo userLoginInfo)
        {
            var phoneLgoin = userLoginInfo;
            if (string.IsNullOrWhiteSpace(phoneLgoin.Phone))
            {
                throw new RpcException((int)CustomerRpcCode.PhoneEmpty, "Phone can't be null", "Phone can't be null");
            }
            var userDTO = await _userServiceProxy.FindUserByPhoneAsync(phoneLgoin.Phone);
            if (userDTO != null)
            {
                _checkedLoginLockAction?.Invoke(userDTO.UserCode);
                await UpgradeLogintimes(userDTO.MappingTo<UserPasswordDTO>());
                var plateform = userLoginInfo.Platform;
                var loginSource = userLoginInfo.LoginSource;
                var userCode = userDTO.UserCode;
                var loginServer = ConfigurationManager.Host;//should be from userLoginInfo
                var ipValue = 0;//should be from userLoginInfo
                var token = await _authenticationServiceProxy.ApplyTokenAsync(new ApplyTokenRequest(AccountType.User, userDTO.UserName, plateform, loginSource, userCode, loginServer, ipValue));
                return token.Code;
            }
            //这里可能需要引入自动注册
            throw new RpcException((int)CustomerRpcCode.PhoneUserNotExisted, "Phone user can't found", "Phone user can't found");
        }

        private async Task<bool> UpgradeLogintimes(UserPasswordDTO userPasswordDTO)
        {
            userPasswordDTO.Logintimes += 1;
            if (userPasswordDTO.Logintimes == 1)
            {
                userPasswordDTO.UserState = UserInfoStateEnum.Activated;
            }
            return await _userServiceProxy.UpdateUserLoginInfoAsync(
      new UpdateUserLoginInfoDBRequest()
      {
          Code = userPasswordDTO.UserCode,
          LoginTimes = userPasswordDTO.Logintimes,
          UserState = userPasswordDTO.UserState,
          LoginLockInfo = new LoginLockInfoDTO()
      });
        }

    }
}
