﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.Interface;
using System.Linq;
using JsonRpcLite.Services;
using WingUserModule.Helper;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.DTO.Organization;
using RemoveOrganizationsRequest = WingInterfaceLibrary.Request.User.RemoveOrganizationsRequest;
using AddOrganizationsRequest = WingInterfaceLibrary.Request.User.AddOrganizationsRequest;
using WingInterfaceLibrary.Request;
using WingServerCommon.Service;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Result.Organization;
using WingInterfaceLibrary.Request.Organization;
using System.Text.RegularExpressions;
using WingServerCommon.Config.Parameters;
using Newtonsoft.Json.Linq;
using System.IO;
using System;
using WingServerCommon.Log;
using WingInterfaceLibrary.Result.Languge;
using WingInterfaceLibrary.Request.Languge;
using WingServerCommon.Mapper;
using WingInterfaceLibrary.DTO.User;
using WingServerCommon.Interfaces.Cache;
using WingInterfaceLibrary.Result;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IOrganizationService
    {

        /// <summary>
        /// 按关键字搜索组织
        /// </summary>
        /// <param name="keyword">关键字</param>
        /// <returns></returns>
        public async Task<List<OrganizationDTO>> SearchOrganizationsAsync(SearchOrganizationsRequest request)
        {
            //验证权限
            var organizations = await _organizationDBService.SearchOrganizationsAsync(request.Keyword, request.ParentCode, (OrganizationTypeEnum)((int)request.OrganizationType));
            if (!string.IsNullOrWhiteSpace(request.ParentCode))
            {
                organizations = organizations.Where(f => f.ParentCode == request.ParentCode).ToList();
            }
            return organizations;
        }

        /// <summary>
        /// 根据组织编码查询组织
        /// </summary>
        /// <param name="organizationCode">组织编码</param>
        /// <returns></returns>
        public async Task<OrganizationDTO> GetOrganizationByCodeAsync(GetOrganizationByCodeRequest request)
        {
            //验证权限
            var organization = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest
            {
                OrganizationCode = request.OrganizationCode
            });

            if (organization != null && !string.IsNullOrEmpty(organization.ParentCode))
            {
                var parentOrganization = CacheMaintenance.Instance.Get<IOrganizationsManager>()?.Get(organization.ParentCode);
                if (parentOrganization != null)
                {
                    organization.ParentOrganizationName = parentOrganization?.OrganizationName;
                }
            }
            //根据添加顺序再排序
            if (organization != null
            && organization.ReferralOrganizationCodes != null && organization.ReferralOrganizationCodes.Count > 0
            && organization.ReferralOrganizations != null && organization.ReferralOrganizations.Count > 0)
            {
                var orderResult = new List<ReferralOrganizationDTO>();
                foreach (var code in organization.ReferralOrganizationCodes)
                {
                    var findOrg = organization.ReferralOrganizations.FirstOrDefault(f => f.OrganizationCode == code);
                    if (findOrg != null)
                    {
                        orderResult.Add(findOrg);
                    }
                }
                organization.ReferralOrganizations = orderResult;
            }
            return organization;
        }

        /// <summary>
        /// 添加组织
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>添加数量</returns>
        public async Task<bool> AddOrganizationsAsync(AddOrganizationsRequest request)
        {
            //验证权限
            if (request != null && request.OrganizationInfos != null && request.OrganizationInfos.Count > 0)
            {
                var organizationEmptyName = request.OrganizationInfos.FirstOrDefault(v => string.IsNullOrWhiteSpace(v.OrganizationName));
                if (organizationEmptyName != null)
                {
                    ThrowCustomerException(CustomerRpcCode.OrganizationNameIsEmpty, "OrganizationName  is Empty");
                }
                if (string.IsNullOrEmpty(request.LanguageCode))
                {
                    ThrowCustomerException(CustomerRpcCode.LanguageCodeIsEmpty, "Language Code  is Empty");
                }
                if (request.LanguageCode == "zh-CN")
                {
                    var organizationLengthName = request.OrganizationInfos.FirstOrDefault(v => v.OrganizationName.Length > 20);
                    if (organizationLengthName != null)
                    {
                        ThrowCustomerException(CustomerRpcCode.NameLengthError, "Name is too long less than 20");
                    }
                }
                else
                {
                    var organizationLengthName = request.OrganizationInfos.FirstOrDefault(v => v.OrganizationName.Length > 50);
                    if (organizationLengthName != null)
                    {
                        ThrowCustomerException(CustomerRpcCode.NameLengthError, "Name is too long less than 50");
                    }
                }
                var patientTypeName = request.OrganizationInfos.FirstOrDefault(v => (int)v.PatientType != 0 && (int)v.PatientType != 1);
                if (patientTypeName != null)
                {
                    ThrowCustomerException(CustomerRpcCode.PatientTypeError, "PatientType code error ");
                }
                var dbRequest = new CreateOrganizationsDBRequest();
                foreach (var org in request.OrganizationInfos)
                {
                    var orgEntity = new OrganizationDTO();
                    var orgCode = CodeCreator.CreateCode("Organization");
                    orgEntity.OrganizationCode = orgCode;
                    orgEntity.OrganizationName = org.OrganizationName;
                    orgEntity.OrganizationType = OrganizationTypeEnum.Corporation;
                    orgEntity.ParentCode = string.Empty;
                    orgEntity.RegionCode = "1000000";
                    orgEntity.RootCode = orgCode;
                    orgEntity.PatientType = org.PatientType;
                    orgEntity.ReferralOrganizationCodes = null;
                    if (!string.IsNullOrWhiteSpace(org.ParentCode))
                    {
                        var parentOrgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = org.ParentCode });
                        if (parentOrgInfo != null)
                        {
                            orgEntity.OrganizationType = parentOrgInfo.OrganizationType + 1;
                            orgEntity.ParentCode = parentOrgInfo.OrganizationCode;
                            orgEntity.RegionCode = parentOrgInfo.RegionCode;
                            orgEntity.RootCode = parentOrgInfo.RootCode;
                        }
                    }
                    dbRequest.Infos.Add(new TableDataItem<OrganizationDTO>
                    {
                        Data = orgEntity,
                        ExtensionData = org.ExtendsData
                    });
                }
                if (dbRequest.Infos.Count > 0)
                {
                    return await _organizationDBService.CreateOrganizationsAsync(dbRequest) > 0;
                }
            }
            return false;
        }

        /// <summary>
        /// 删除组织
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>删除数量</returns>
        public async Task<bool> RemoveOrganizationsAsync(RemoveOrganizationsRequest request)
        {
            //验证权限
            var dbRequest = new RemoveOrganizationsDBRequest();
            dbRequest.OrganizationCodes = request.OrganizationCodes;
            var orgInfos = await _organizationDBService.FindOrganizationListByCodesAsync(request.OrganizationCodes);
            var users = await _userDBService.FindUserListByOrganizationCodesAsync(request.OrganizationCodes);
            foreach (var orgCode in request.OrganizationCodes)
            {
                var orgInfo = orgInfos.FirstOrDefault(o => o.OrganizationCode == orgCode);
                var orgUsers = users.Where(u => u.OrganizationCode == orgCode);
                foreach (var item in orgUsers)
                {
                    var userPasswordDTO = item.MappingTo<UserPasswordDTO>();
                    userPasswordDTO.OrganizationCode = orgInfo?.RootCode;
                    await _userDBService.UpdateUserAsync(userPasswordDTO.UserCode, userPasswordDTO, null, true);
                }
            }

            return await _organizationDBService.RemoveOrganizationsAsync(dbRequest) > 0;
        }

        /// <summary>
        /// 查询上级机构列表
        /// </summary>
        /// <param name="token">权鉴token</param>
        /// <returns>上级机构列表</returns>
        public async Task<List<OrganizationBaseDTO>> GetParentOrganizationListAsync(GetParentOrganizationListRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            string organizationCode = "";
            //获取机构编码            
            var userInfoDO = await _userDBService.FindUserByCodeAsync(userCode);
            if (userInfoDO != null && !string.IsNullOrEmpty(userInfoDO.RootOrganizationCode))
            {
                organizationCode = userInfoDO.RootOrganizationCode;
            }
            var resultList = new List<OrganizationBaseDTO>();
            if (!string.IsNullOrEmpty(organizationCode))
            {
                var organizationList = await _organizationDBService.FindAllOrganizationByTypeAsync(organizationCode);
                if (organizationList != null && organizationList.Any())
                {
                    resultList = organizationList.Select(c => new OrganizationBaseDTO()
                    {
                        OrganizationCode = c.OrganizationCode,
                        OrganizationName = c.OrganizationName,
                    }).ToList();
                }
            }
            return resultList;
        }

        /// <summary>
        /// 查询机构列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>机构列表</returns>
        public async Task<PageResult<OrganizationBaseDTO>> FindOrganizationsByPageAsync(FindOrganizationListRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfoDO = await _userDBService.FindUserByCodeAsync(userCode);

            var pageIndex = request.PageIndex - 1;
            var pageSize = request.PageSize;
            var result = new PageResult<OrganizationBaseDTO>
            {
                PageIndex = request.PageIndex,
                PageSize = pageSize,
                TotalCount = 0,
                PageData = new List<OrganizationBaseDTO>()
            };
            if (userInfoDO != null && !string.IsNullOrEmpty(userInfoDO.RootOrganizationCode))
            {
                var organization = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfoDO.RootOrganizationCode);
                var organizationList = await _organizationDBService.GetOrganizationListAsync(new GetOrganizationListDBRequest
                {
                    OrganizationType = OrganizationTypeEnum.Corporation,
                    PatientType = (OrganizationPatientTypeEnum)organization.PatientType,
                    Isinvented = false,
                });
                if (!string.IsNullOrWhiteSpace(request.KeyWord))
                {
                    organizationList = organizationList.Where(x => x.OrganizationName.Contains(request.KeyWord))?.ToList() ?? new List<OrganizationDTO>();
                }
                result.TotalCount = organizationList.Count;
                organizationList = organizationList.Skip(pageIndex * pageSize).Take(pageSize).ToList();
                result.PageData = organizationList.Select(c => new OrganizationBaseDTO()
                {
                    OrganizationCode = c.OrganizationCode,
                    OrganizationName = c.OrganizationName,
                }).ToList();
            }
            return result;
        }

        /// <summary>
        /// 查询上级机构列表
        /// </summary>
        /// <param name="request">请求对象</param>
        /// <returns>机构列表</returns>
        public async Task<PageResult<OrganizationBaseDTO>> FindPatientOrganizationsByPageAsync(FindPatientOrganizationsByPageRequest request)
        {
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userInfoDO = await _userDBService.FindUserByCodeAsync(userCode);

            var pageIndex = request.PageIndex - 1;
            var pageSize = request.PageSize;
            var result = new PageResult<OrganizationBaseDTO>
            {
                PageIndex = request.PageIndex,
                PageSize = pageSize,
                TotalCount = 0,
                PageData = new List<OrganizationBaseDTO>()
            };
            if (userInfoDO != null)
            {
                var organization = CacheMaintenance.Instance.Get<IOrganizationsManager>().Get(userInfoDO.RootOrganizationCode);
                var organizationList = await _organizationDBService.GetOrganizationListAsync(new GetOrganizationListDBRequest
                {
                    OrganizationType = OrganizationTypeEnum.Corporation,
                    Isinvented = false,
                });
                if (!string.IsNullOrWhiteSpace(request.KeyWord))
                {
                    organizationList = organizationList.Where(x => x.OrganizationName.Contains(request.KeyWord))?.ToList() ?? new List<OrganizationDTO>();
                }
                organizationList = organizationList.Where(x => x.OrganizationCode != userInfoDO.RootOrganizationCode)?.ToList() ?? new List<OrganizationDTO>();
                result.TotalCount = organizationList.Count;
                organizationList = organizationList.Skip(pageIndex * pageSize).Take(pageSize).ToList();
                result.PageData = organizationList.Select(c => new OrganizationBaseDTO()
                {
                    OrganizationCode = c.OrganizationCode,
                    OrganizationName = c.OrganizationName,
                }).ToList();
            }
            return result;
        }

        /// <summary>
        /// 查询机构详细信息 根据 机构编码
        /// </summary>
        /// <param name="token">权鉴token</param>
        /// <returns>机构详细信息</returns>
        public async Task<OrganizationBasicDTO> GetPersonOrganizationAsync(GetPersonOrganizationRequest request)
        {
            //验证权限
            var userCode = await GetClientIdByTokenAsync(request.Token);
            string organizationCode = "";
            //获取机构编码           
            var userInfoDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (userInfoDTO != null && !string.IsNullOrEmpty(userInfoDTO.OrganizationCode))
            {
                organizationCode = userInfoDTO.OrganizationCode;
            }
            if (string.IsNullOrEmpty(organizationCode))
            {
                ThrowRpcException((int)CustomerRpcCode.UserOrganizationCodeEmpty, "Organization Code Empty error");
            }
            var organization = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest
            {
                OrganizationCode = organizationCode
            });
            var result = new OrganizationBasicDTO();
            if (organization != null)
            {
                result.LogoUrl = organization.LogoUrl;
                result.OrganizationCode = organization.OrganizationCode;
                result.OrganizationName = organization.OrganizationName;
                result.ParentCode = organization.ParentCode;
                result.RegionCode = organization.RegionCode;
                result.PatientType = organization.PatientType;
                result.DiagnosisModules = organization.DiagnosisModules;
                result.IsEncryptedShow = organization.IsEncryptedShow;
                result.ReferralLimitHours = organization.ReferralLimitHours;
                result.ReferralOrganizationCodes = organization.ReferralOrganizationCodes;
                result.ReferralOrganizations = organization.ReferralOrganizations;
            }
            return result;
        }

        /// <summary>
        /// 保存机构信息
        /// </summary>
        /// <param name="token">权鉴token</param>
        /// <param name="model">机构基础信息对象</param>
        /// <returns>是否成功</returns>
        public async Task<bool> SavePersonOrganizationAsync(SavePersonOrganizationRequest request)
        {
            //验证权限
            if (string.IsNullOrEmpty(request.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.OrganizationCodeIsEmpty, "OrganizationCode Empty Error");
            }
            if (string.IsNullOrEmpty(request.OrganizationName))
            {
                ThrowCustomerException(CustomerRpcCode.OrganizationNameIsEmpty, "OrganizationName Empty Error");
            }
            var organization = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest
            {
                OrganizationCode = request.OrganizationCode
            }) ?? new OrganizationDTO();
            if (string.IsNullOrEmpty(organization.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.UserOrganizationCodeEmpty, "Organization Code Empty error");
            }
            //查到该机构所有的子机构
            var needOrganizations = new List<OrganizationDTO>();
            GetAllChildOrganizationDTO(organization, ref needOrganizations);
            if (needOrganizations.Select(n => n.OrganizationCode).Contains(request.ParentCode))
            {
                ThrowCustomerException(CustomerRpcCode.UserOrgCanNotChoseChild, "Parent organization can not choose Sub level organization");
            }
            organization.OrganizationName = request.OrganizationName;
            organization.ParentCode = request.ParentCode;
            if (!string.IsNullOrEmpty(request.RegionCode))
            {
                organization.RegionCode = request.RegionCode;
            }
            if (!string.IsNullOrEmpty(request.LogoUrl))
            {
                organization.LogoUrl = request.LogoUrl.ToUrlToken();
            }
            //修改服务对象将清除自定义配置
            if (organization.PatientType != request.PatientType)
            {
                organization.PatientType = request.PatientType;
                organization.PatientSettingJson = "";
                organization.PatientSettingVersion = "";
            }
            organization.IsEncryptedShow = request.IsEncryptedShow;
            organization.ReferralLimitHours = request.ReferralLimitHours;
            organization.ReferralOrganizationCodes = request.ReferralOrganizationCodes ?? new List<string>();
            var result = await _organizationDBService.SaveOrganizationAsync(organization.OrganizationCode, organization, true, true);
            return result;
        }

        /// <summary>
        /// 更新组织名称
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateOrganizationNameAsync(UpdateOrganizationNameRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.OrganizationCode))
            {
                ThrowCustomerException(CustomerRpcCode.OrganizationCodeIsEmpty, "OrganizationCode can't be empty");
            }
            if (string.IsNullOrWhiteSpace(request.OrganizationName))
            {
                ThrowCustomerException(CustomerRpcCode.OrganizationNameIsEmpty, "OrganizationName can't be empty");
            }
            return await _organizationDBService.UpdateOrganizationNameAsync(new UpdateOrganizationNameDBRequest
            {
                OrganizationCode = request.OrganizationCode,
                OrganizationName = request.OrganizationName
            });
        }

        /// <summary>
        /// 客户端获取机构配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<OrganizationSettingResult> GetOrganizationSettingAsync(GetUserOrganizationSettingRequest request)
        {
            //验证权限
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            if (!Regex.IsMatch(request.Version, "^[0-9]{1}(.[0-9])+$"))
            {
                ThrowCustomerException(CustomerRpcCode.UserVersionInValid, "Not real version");
            }
            var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = userDTO.RootOrganizationCode });
            var json = "";
            var returnVersion = "";
            switch (request.SettingType)
            {
                case OrganizationSettingTypeEnum.Patient:
                    var patientResult = GetPatientSetting(orgInfo, request);
                    json = patientResult.Item1;
                    returnVersion = patientResult.Item2;
                    break;
                case OrganizationSettingTypeEnum.Exam:
                    var examResult = GetExamSetting(orgInfo, request);
                    json = examResult.Item1;
                    returnVersion = examResult.Item2;
                    break;
                case OrganizationSettingTypeEnum.Consultation:
                    var consultationResult = GetConsultationSetting(orgInfo, request);
                    json = consultationResult.Item1;
                    returnVersion = consultationResult.Item2;
                    break;
                default:
                    break;
            }
            var result = new OrganizationSettingResult { SettingVersion = returnVersion, SettingData = json };
            return result;
        }

        private (string, string) GetPatientSetting(OrganizationDTO orgInfo, GetUserOrganizationSettingRequest request)
        {
            var json = "";
            var returnVersion = "";
            if (!string.IsNullOrWhiteSpace(orgInfo.PatientSettingJson))
            {
                if (Version.Parse(orgInfo.PatientSettingVersion) > Version.Parse(request.Version))
                {
                    json = orgInfo.PatientSettingJson;
                    returnVersion = orgInfo.PatientSettingVersion;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
                }
            }
            else
            {
                var baseJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Exam", orgInfo.PatientType == OrganizationPatientTypeEnum.Person ? "PatientInfo.json" : "AnimalInfo.json"));
                JObject o = JObject.Parse(baseJson);
                var personDefaultVersion = o["General"]["Version"].ToString();
                if (Version.Parse(personDefaultVersion) > Version.Parse(request.Version))
                {
                    o.Remove("General");
                    json = o.ToString();
                    returnVersion = personDefaultVersion;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
                }
            }
            return (json, returnVersion);
        }

        private (string, string) GetExamSetting(OrganizationDTO orgInfo, GetUserOrganizationSettingRequest request)
        {
            var json = "";
            var returnVersion = "";
            if (!string.IsNullOrWhiteSpace(orgInfo.ExamSettingJson))
            {
                if (Version.Parse(orgInfo.ExamSettingVersion) > Version.Parse(request.Version))
                {
                    json = orgInfo.ExamSettingJson;
                    returnVersion = orgInfo.ExamSettingVersion;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
                }
            }
            else
            {
                var baseJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Exam", "ExamInfo.json"));
                JObject o = JObject.Parse(baseJson);
                var defaultVersion = o["General"]["Version"].ToString();
                if (Version.Parse(defaultVersion) > Version.Parse(request.Version))
                {
                    o.Remove("General");
                    json = o.ToString();
                    returnVersion = defaultVersion;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
                }
            }
            return (json, returnVersion);
        }

        private (string, string) GetConsultationSetting(OrganizationDTO orgInfo, GetUserOrganizationSettingRequest request)
        {
            var json = "";
            var returnVersion = "";
            if (!string.IsNullOrWhiteSpace(orgInfo.ConsultationSettingJson))
            {
                if (Version.Parse(orgInfo.ConsultationSettingVersion) > Version.Parse(request.Version))
                {
                    json = orgInfo.ConsultationSettingJson;
                    returnVersion = orgInfo.ConsultationSettingVersion;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
                }
            }
            else
            {
                var baseJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Consultation", "ConsultationInfo.json"));
                JObject o = JObject.Parse(baseJson);
                var defaultVersion = o["General"]["Version"].ToString();
                if (Version.Parse(defaultVersion) > Version.Parse(request.Version))
                {
                    o.Remove("General");
                    json = o.ToString();
                    returnVersion = defaultVersion;
                }
                else
                {
                    ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
                }
            }
            return (json, returnVersion);
        }

        /// <summary>
        /// 客户端获取Server配置
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<ServerSettingResult> GetServerSettingAsync(TokenRequest request)
        {
            var result = new ServerSettingResult();
            //setting配置
            result.ServerConfigList.Add("HeartRateSeconds", _heartRateSeconds.ToString());
            result.ServerConfigList.Add("DefaultQueryDays", _defaultQueryDays.ToString());
            result.ServerConfigList.Add("NotificationUrl", _webSocketUrl);
            result.ServerConfigList.Add("LiveConsultationRateSeconds", _liveConsultationRateSeconds.ToString());
            result.ServerConfigList.Add("CMSUrl", _cmsUrl);
            result.HeartRateSeconds = _heartRateSeconds;
            result.LiveConsultationDefaultQueryDays = _defaultQueryDays;
            result.NotificationUrl = _webSocketUrl;
            result.LiveConsultationRateSeconds = _liveConsultationRateSeconds;
            result.CMSUrl = _cmsUrl;
            //验证权限
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            DirectoryInfo theFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge"));
            DirectoryInfo[] dirInfo = theFolder.GetDirectories();//获取所在目录的文件夹
            foreach (var item in dirInfo)
            {
                var serverLanguge = new ServerLangugeClass();
                serverLanguge.LangugeKey = item.Name;
                DirectoryInfo fileFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", item.Name));
                FileInfo[] file = fileFolder.GetFiles();//获取所在目录的文件
                foreach (FileInfo fileItem in file) //遍历文件
                {
                    serverLanguge.LangugeModules.Add(fileItem.Name);
                }
                result.ServerLangugeList.Add(serverLanguge);
            }
            return result;
        }

        /// <summary>
        /// 根据语言key获取该语言下的配置列表
        /// </summary>
        /// <param name="langugeKey"></param>
        /// <returns></returns>
        private List<string> GetLangugeModules(string langugeKey)
        {
            var list = new List<string>();
            DirectoryInfo fileFolder = new DirectoryInfo(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Languge", langugeKey));
            FileInfo[] file = fileFolder.GetFiles();//获取所在目录的文件
            foreach (FileInfo fileItem in file) //遍历文件
            {
                list.Add(fileItem.Name);
            }
            return list;
        }

        /// <summary>
        /// 获取语言配置列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<List<LangugeDataResult>> GetLangugeDataListAsync(LangugeDataRequest request)
        {
            var result = new List<LangugeDataResult>();
            //验证权限
            var userCode = await GetClientIdByTokenAsync(request.Token);
            var userDTO = await _userDBService.FindUserByCodeAsync(userCode);
            if (userDTO == null)
            {
                ThrowCustomerException(CustomerRpcCode.UserNotExisted, "User not found");
            }
            var langugeModules = GetLangugeModules(request.LangugeKey);
            request.LangugeDataItemList.ForEach(l => l.LangugeModule = l.LangugeModule.EndsWith(".json") ? l.LangugeModule : (l.LangugeModule + ".json"));
            foreach (var langugeModule in langugeModules)
            {
                var version = request.LangugeDataItemList.FirstOrDefault(l => l.LangugeModule == langugeModule)?.Version ?? "0.9";
                var langugeDataItem = await GetLangugeData(userDTO.RootOrganizationCode, request.LangugeKey, langugeModule, version);
                result.Add(langugeDataItem);
            }
            return result;
        }

        /// <summary>
        /// 客户端获取语言配置
        /// </summary>
        /// <param name="orgCode">机构code</param>
        /// <param name="langugeModule">模块（配置文件名）</param>
        /// <param name="version">模板配置版本</param>
        /// <returns></returns>
        private async Task<LangugeDataResult> GetLangugeData(string orgCode, string langugeKey, string langugeModule, string version)
        {
            var result = new LangugeDataResult { LangugeModule = langugeModule.Replace(".json", "") };
            try
            {
                if (!Regex.IsMatch(version, "^[0-9]{1}(.[0-9])+$"))
                {
                    version = "0.9";
                }
                //在取值时判断是否基础配置的版本并升级
                var orgInfo = await _organizationDBService.GetOrganizationDetailAsync(new GetOrganizationDBRequest { OrganizationCode = orgCode });
                SettingDTOClass settingObj = null;
                if (!string.IsNullOrWhiteSpace(orgCode))
                {
                    settingObj = orgInfo.Settings.FirstOrDefault(s => s.SettingModule == langugeModule && s.LangugeKey == langugeKey);
                }
                //默认配置
                var baseJson = WingServerCommon.ResourceManage.ResourceManager.GetSetting(langugeKey + langugeModule);
                if (string.IsNullOrWhiteSpace(baseJson))
                {
                    return result;
                }
                JObject o = JObject.Parse(baseJson);
                var defaultVersion = o["General"]["Version"].ToString();

                //测试
                //WingServerCommon.ResourceManage.ResourceManager.ReLoadSetting(langugeKey + langugeModule);
                // o.Remove("General");
                // JObject language = (JObject)o["Language"];
                // //拼接自定义json
                // var customContentJson = JObject.Parse("{\"Name1\": \"张三\",\"Phone1\":\"188\"}");
                // foreach (var item in customContentJson)
                // {
                //     JProperty jp = new JProperty(item.Key, item.Value);
                //     language.Add(jp);
                // }
                // json = o.ToString();

                //无自定义配置
                if (settingObj == null || string.IsNullOrWhiteSpace(settingObj.CustomContent))
                {
                    if (Version.Parse(defaultVersion) > Version.Parse(version))
                    {
                        o.Remove("General");
                        result.Content = o.ToString();
                        result.Version = defaultVersion;
                    }
                    else
                    {
                        result.Version = version;
                    }
                    return result;
                }
                //有自定义配置
                //当该自定义配置基于的默认配置版本落后于当前的默认配置版本时
                if (Version.Parse(defaultVersion) > Version.Parse(settingObj.BaseConfigVersion))
                {
                    o.Remove("General");
                    settingObj.BaseConfigVersion = defaultVersion;
                    settingObj.Version = (double.Parse(settingObj.Version) + 0.1).ToString("f1");
                    JObject language = (JObject)o["Language"];
                    //拼接自定义json
                    var customContentJson = JObject.Parse(settingObj.CustomContent);
                    foreach (var item in customContentJson)
                    {
                        language.Add(new JProperty(item.Key, item.Value));
                    }
                    //更新完整内容
                    settingObj.CompleteContent = o.ToString();
                    await _organizationDBService.SaveOrganizationAsync(orgInfo.OrganizationCode, orgInfo);
                }
                //拼接自定义和默认配置
                if (Version.Parse(settingObj.Version) > Version.Parse(version))
                {
                    result.Content = settingObj.CompleteContent;
                    result.Version = settingObj.Version;
                }
                else
                {
                    result.Version = settingObj.Version;
                }
            }
            catch (Exception ex)
            {
                Logger.WriteLineWarn($"GetLangugeData err:request: orgCode-{orgCode},langugeKey-{langugeKey},langugeModule-{langugeModule},version-{version},error:{ex},{ex.StackTrace}");
            }
            return result;
        }

        /// <summary>
        /// 取该医院组织的子孙医院
        /// </summary>
        /// <param name="organizations">全部组织</param>
        /// <param name="organizationCode">当前组织Code</param>
        /// <param name="needOrganizations">全部子孙组织</param>
        /// <returns></returns>
        private void GetAllChildOrganizationDTO(OrganizationDTO organization, ref List<OrganizationDTO> needOrganizations)
        {
            var organizationCode = organization.OrganizationCode;
            //查到父级为此机构的机构列表
            var organizationFilter = new Dictionary<string, string>();
            organizationFilter.Add("ParentCode", organizationCode);
            organizationFilter.Add("OrganizationType", OrganizationTypeEnum.Corporation.ToString());
            var organizations = _organizationDBService.GetOrganizationPageAsync((new PageFilterRequest { CurrentPage = 1, PageSize = 999999, Filter = organizationFilter, IsFuzzy = false })).Result;
            var childList = organizations.PageData.Where(o => o.ParentCode == organizationCode && o.OrganizationCode != organizationCode);
            if (childList != null && childList.Count() > 0)
            {
                var intersectedList = needOrganizations.Select(n => n.OrganizationCode).Intersect(childList.Select(c => c.OrganizationCode));
                //避免循环设置上级：a->b->c->a
                if (intersectedList.Count() == 0)
                {
                    needOrganizations.AddRange(childList);
                    foreach (var item in childList)
                    {
                        GetAllChildOrganizationDTO(item, ref needOrganizations);
                    }
                }

            }
        }
    }
}
