using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using JsonRpcLite.Services;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.DTO.Position;
using WingInterfaceLibrary.Enum;
using WingInterfaceLibrary.Interface;
using WingInterfaceLibrary.Interface.DBInterface;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Request.User;
using WingServerCommon.Service;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IPositionService
    {
        private IPositionDBService _positionDBService;
        /// <summary>
        /// 添加岗位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>添加数量</returns>
        public async Task<bool> AddPositionsAsync(WingInterfaceLibrary.Request.User.AddPositionsRequest request)
        {
            var positionEmptyName = request.Positions.FirstOrDefault(v => string.IsNullOrWhiteSpace(v.PositionName));
            if (positionEmptyName != null)
            {
                ThrowCustomerException(CustomerRpcCode.PositionNameIsEmpty, "PositionName is Empty");
            }
            var dbRequest = new CreatePositionsDBRequest();
            dbRequest.Infos = request.Positions.Select(f=>
                new TableDataItem<PositionDTO>
                { 
                    Data = new PositionDTO 
                    { 
                        UnderUserCodes = f.UnderUserCodes,
                        PositionName = f.PositionName, 
                        OrganizationCode = f.OrganizationCode,
                        FitDepartmentCode = f.FitDepartmentCode
                    }, 
                    ExtensionData = f.ExtendsData
                }).ToList();
            return await _positionDBService.AddPositionsAsync(dbRequest)>0;
        }

        /// <summary>
        /// 获取岗位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>岗位信息列表</returns>
        public async Task<List<PositionDTO>> GetPositionsAsync(WingInterfaceLibrary.Request.User.GetPositionsRequest request)
        {
            var dbRequest = new GetPositionsDBRequest();
            dbRequest.OrganizationCodes = request.OrganizationCodes;
            dbRequest.PositionCodes = request.PositionCodes;
            dbRequest.FitDepartmentCode = request.FitDepartmentCode;
            var result = await _positionDBService.GetPositionsAsync(dbRequest);
            return result;
        }

        /// <summary>
        /// 删除岗位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>删除数量</returns>
        public async Task<bool> RemovePositionsAsync(WingInterfaceLibrary.Request.User.RemovePositionRequest request)
        {
            var dbRequest = new RemovePositionDBRequest();
            dbRequest.PositionCodes = request.PositionCodes;
            return await _positionDBService.RemovePositionsAsync(dbRequest) >0;
        }

        /// <summary>
        /// 更新岗位角色
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdatePositionNameAsync(UpdatePositionNameRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.PositionCode))
            {
                ThrowCustomerException(CustomerRpcCode.PositionCodeIsEmpty, "PositionCode can't be empty");
            }
            if (string.IsNullOrWhiteSpace(request.PositionName))
            {
                ThrowCustomerException(CustomerRpcCode.PositionNameIsEmpty, "PositionName can't be empty");
            }
            if (string.IsNullOrWhiteSpace(request.FitDepartmentCode))
            {
                ThrowCustomerException(CustomerRpcCode.FitDepartmentCodeIsEmpty, "FitDepartmentCode can't be empty");
            }
            return await _positionDBService.UpdatePositionNameAsync(new UpdatePositionNameDBRequest{
                PositionCode = request.PositionCode,
                PositionName = request.PositionName,
                FitDepartmentCode = request.FitDepartmentCode
            });
        }
    }
}