﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.Interface;
using JsonRpcLite.Services;
using System.Linq;
using WingInterfaceLibrary.Request.User;
using GetRanksRequest = WingInterfaceLibrary.Request.User.GetRanksRequest;
using AddRanksRequest = WingInterfaceLibrary.Request.User.AddRanksRequest;
using RemoveRanksRequest = WingInterfaceLibrary.Request.User.RemoveRanksRequest;
using WingInterfaceLibrary.DTO.Rank;
using WingInterfaceLibrary.Request;
using WingServerCommon.Service;
using WingInterfaceLibrary.DB.Request;
using WingInterfaceLibrary.Enum;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IRankService
    {

        /// <summary>
        /// 获取职位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>职位信息列表</returns>
        public async Task<RankDTO> GetRankByCodeAsync(GetRankByCodeRequest request)
        {
            var rank = await _rankDBService.GetRankByCodeAsync(request.RankCode);
            return rank;
        }

        /// <summary>
        /// 获取所有职位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>职位信息列表</returns>
        public async Task<List<RankDTO>> GetAllRanksAsync(GetRanksRequest request)
        {
            var ranks = await _rankDBService.GetAllRanksAsync(new GetRanksDBRequest
            {
                RankCodes = request.RankCodes,
                OrganizationCodes = request.OrganizationCodes
            });
            return ranks;
        }

        /// <summary>
        /// 添加职位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>添加数量</returns>
        public async Task<bool> AddRanksAsync(AddRanksRequest request)
        {
            var rankEmptyName = request.RankInfos.FirstOrDefault(v => string.IsNullOrWhiteSpace(v.RankName));
            if (rankEmptyName != null)
            {
                ThrowCustomerException(CustomerRpcCode.RankNameIsEmpty, "RankName is Empty");
            }
            var dbRequest = new CreateRanksDBRequest();
            dbRequest.Infos = request.RankInfos.Select(f=>
                new TableDataItem<RankDTO>
                { 
                    Data = new RankDTO { RankName = f.RankName, OrganizationCode = f.OrganizationCode }, 
                    ExtensionData = f.ExtendsData
                }).ToList();
            return await _rankDBService.AddRanksAsync(dbRequest) > 0;
        }

        /// <summary>
        /// 删除职位信息
        /// </summary>
        /// <param name="request">请求体</param>
        /// <returns>删除数量</returns>
        public async Task<bool> RemoveRanksAsync(RemoveRanksRequest request)
        {
            return await _rankDBService.RemoveRanksAsync(new RemoveRanksDBRequest
            {
                RankCodes = request.RankCodes
            }) > 0;
        }

        /// <summary>
        /// 更新职称名称
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        public async Task<bool> UpdateRankNameAsync(UpdateRankNameRequest request)
        {
            if (string.IsNullOrWhiteSpace(request.RankCode))
            {
                ThrowCustomerException(CustomerRpcCode.RankCodeIsEmpty, "RankCode can't be empty");
            }
            if (string.IsNullOrWhiteSpace(request.RankName))
            {
                ThrowCustomerException(CustomerRpcCode.RankNameIsEmpty, "RankName can't be empty");
            }
            return await _rankDBService.UpdateRankNameAsync(new UpdateRankNameDBRequest{
                RankCode = request.RankCode,
                RankName = request.RankName
            });
        }
    }
}
