﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.Interface;
using System.IO;
using System;
using System.Text.Json.Serialization;
using JsonRpcLite.Services;
using System.Linq;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.DTO.Region;
using System.Text.RegularExpressions;
using WingUserModule.Serialization;
using WingServerCommon.Service;
using WingInterfaceLibrary.Enum;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IRegionService
    {
        /// <summary>
        /// 获取国家地区信息
        /// </summary>
        /// <param name="version">版本</param>
        /// <param name="languageType">语言</param>
        /// <returns>国家地区信息</returns>
        public Task<RegionDTO> GetRegionsAsync(GetRegionsRequest request)
        {
            var regionJson = File.ReadAllText(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Resource", "Region", "RegionTable.json"));
            var regionData = JsonHelper.DeserializerJson<RegionData>(regionJson);
            RegionDTO regionDTO = null;
            if (!Regex.IsMatch(request.Version, "^[0-9]{1}(.[0-9])+$"))
            {
                ThrowCustomerException(CustomerRpcCode.UserVersionInValid, "Not real version");
            }
            if (Version.Parse(regionData.Version) > Version.Parse(request.Version))
            {
                var datas = regionData.RegionCollections.FirstOrDefault(f => f.LangCode == request.LanguageType);
                if (datas != null)
                {
                    regionDTO = new RegionDTO();
                    regionDTO.LanguageType = request.LanguageType;
                    regionDTO.RegionVersion = regionData.Version;
                    regionDTO.ReginData = JsonHelper.SerializerToJson(regionData.RegionCollections.FirstOrDefault(f => f.LangCode == request.LanguageType));
                    return Task.FromResult(regionDTO);
                }
                ThrowCustomerException(CustomerRpcCode.ReginDataEmpty, "Can't find region data by lang code");
            }
            ThrowCustomerException(CustomerRpcCode.UserGreaterThanNewestVersion, "This is the latest version");
            return null;
        }

        class RegionData
        {
            [JsonPropertyName("ver")]
            public string Version { get; set; }

            [JsonPropertyName("rc")]
            public List<RegionCollection> RegionCollections { get; set; } = new();
        }

        class RegionCollection
        {
            [JsonPropertyName("lang")]
            public string LangCode { get; set; }

            [JsonPropertyName("datas")]
            public List<RegionItem> RegionItems { get; set; } = new();
        }

        class RegionItem
        {
            [JsonPropertyName("c")]
            public string RegionCode { get; set; }

            [JsonPropertyName("g")]
            public string RegionGrade { get; set; }
            [JsonPropertyName("f")]
            public string FatherCode { get; set; }
            [JsonPropertyName("n")]
            public string Name { get; set; }
        }
    }
}
