﻿using System.Threading.Tasks;
using System.Collections.Generic;
using WingInterfaceLibrary.Interface;
using JsonRpcLite.Services;
using WingInterfaceLibrary.Request.User;
using WingInterfaceLibrary.DTO.Role;
using WingInterfaceLibrary.Request;
using WingInterfaceLibrary.Enum;
using WingServerCommon.Service;
using System.Linq;

namespace WingUserModule.Service
{
    public partial class UserService : JsonRpcService, IRoleService
    {

        public async Task<RoleDTO> GetRoleByCodeAsync(GetRoleByCodeRequest request)
        {
            var role = await _roleDBService.FindRoleByCodeAsync(request.RoleCode);
            ChangeRoleByLanguageConfig(role, request.Language);
            return role;
        }

        public async Task<List<RoleDTO>> FindDefaultRolesAsync(FindDefaultRolesRequest request)
        {
            var roles = await _roleDBService.FindRolesByTypeAsync(RoleShowTypeEnum.NotShow, request.OrganizationCode);
            foreach (var role in roles)
            {
                ChangeRoleByLanguageConfig(role, request.Language);
            }
            return roles;
        }

        public async Task<List<RoleDTO>> FindAuthenticationRolesAsync(FindAuthenticationRolesRequest request)
        {
            var roles = await _roleDBService.FindRolesByTypeAsync(RoleShowTypeEnum.ISShow, "");
            foreach (var role in roles)
            {
                ChangeRoleByLanguageConfig(role, request.Language);
            }
            return roles;
        }

        private void ChangeRoleByLanguageConfig(RoleDTO role, string areaCode)
        {
            var langConfig = role.LanguageConfigs?.FirstOrDefault(x => x.Language == areaCode);
            if (langConfig != null)
            {
                role.RoleName = langConfig.RoleName;
                role.Description = langConfig.Description;
                role.FieldList = langConfig.FieldList;
            }
            else
            {
                role.RoleName = "";
                role.Description = "";
                role.FieldList = new List<string>();
            }
        }
    }
}
